@@ -8,10 +8,10 @@ import click
 import typer
 from typer.core import TyperGroup
 
-from ops2deb import __version__, builder, formatter, generator, logger, parser, updater
+from ops2deb import __version__, builder, formatter, generator, logger, updater
 from ops2deb.exceptions import Ops2debError
 from ops2deb.fetcher import DEFAULT_CACHE_DIRECTORY, Fetcher
-from ops2deb.migrate import migrate_configuration_file
+from ops2deb.parser import Configuration
 
 
 class DefaultCommandGroup(TyperGroup):
@@ -148,11 +148,11 @@ def default(
     workers_count: int = option_workers_count,
 ) -> None:
     try:
-        fetcher = Fetcher(cache_directory, lockfile_path)
-        blueprints = parser.parse(configuration_path)
+        configuration = Configuration(configuration_path)
+        fetcher = Fetcher(cache_directory, configuration.lockfile_path or lockfile_path)
         packages = generator.generate(
             fetcher,
-            blueprints,
+            configuration.blueprints,
             output_directory,
             configuration_path.parent,
             debian_repository,
@@ -175,11 +175,11 @@ def generate(
     only: Optional[List[str]] = option_only,
 ) -> None:
     try:
-        fetcher = Fetcher(cache_directory, lockfile_path)
-        blueprints = parser.parse(configuration_path)
+        configuration = Configuration(configuration_path)
+        fetcher = Fetcher(cache_directory, configuration.lockfile_path or lockfile_path)
         generator.generate(
             fetcher,
-            blueprints,
+            configuration.blueprints,
             output_directory,
             configuration_path.parent,
             debian_repository,
@@ -230,11 +230,22 @@ def update(
         envvar="OPS2DEB_SKIP_BLUEPRINTS",
         help="Name of blueprint that should not be updated. Can be used multiple times.",
     ),
+    only: Optional[List[str]] = option_only,
+    max_versions: int = typer.Option(
+        1, "-m", "--max-versions", envvar="OPS2DEB_MAX_VERSIONS"
+    ),
 ) -> None:
     try:
-        fetcher = Fetcher(cache_directory, lockfile_path)
+        configuration = Configuration(configuration_path)
+        fetcher = Fetcher(cache_directory, configuration.lockfile_path or lockfile_path)
         updater.update(
-            configuration_path, lockfile_path, fetcher, dry_run, output_path, skip
+            configuration,
+            fetcher,
+            dry_run,
+            output_path,
+            skip,
+            only,
+            max_versions,
         )
     except Ops2debError as e:
         error(e, exit_code)
@@ -247,7 +258,7 @@ def validate(
     configuration_path: Path = option_configuration,
 ) -> None:
     try:
-        parser.parse(configuration_path)
+        Configuration(configuration_path)
     except Ops2debError as e:
         error(e, exit_code)
 
@@ -278,24 +289,12 @@ def lock(
     cache_directory: Path = option_cache_directory,
 ) -> None:
     try:
-        fetcher = Fetcher(cache_directory, lockfile_path)
+        configuration = Configuration(configuration_path)
+        fetcher = Fetcher(cache_directory, configuration.lockfile_path or lockfile_path)
         fetcher.update_lockfile(configuration_path)
     except Ops2debError as e:
         error(e, exit_code)
 
 
-@app.command(help="Move file sha256 hashes to dedicated lockfile")
-def migrate(
-    verbose: bool = option_verbose,
-    exit_code: int = option_exit_code,
-    configuration_path: Path = option_configuration,
-    lockfile_path: Path = option_lockfile,
-) -> None:
-    try:
-        migrate_configuration_file(configuration_path, lockfile_path)
-    except Ops2debError as e:
-        error(e, exit_code)
-
-
 def main() -> None:
     app()
