@@ -1,60 +1,24 @@
 import asyncio
-import bz2
-import gzip
 import hashlib
 import shutil
-import tarfile
 from dataclasses import dataclass
 from pathlib import Path
 from typing import Sequence
 
 import aiofiles
 import httpx
-import unix_ar
 
 from ops2deb import logger
 from ops2deb.client import client_factory
-from ops2deb.exceptions import Ops2debError, Ops2debExtractError, Ops2debFetcherError
+from ops2deb.exceptions import Ops2debError, Ops2debFetcherError
+from ops2deb.extracter import extract_archive, is_archive_format_supported
 from ops2deb.lockfile import Lock
-from ops2deb.parser import RemoteFile, extend, parse
+from ops2deb.parser import Configuration
 from ops2deb.utils import log_and_raise, separate_results_from_errors
 
 DEFAULT_CACHE_DIRECTORY = Path("/tmp/ops2deb_cache")
 
 
-def _unpack_gz(file_path: str, extract_path: str) -> None:
-    output_path = Path(extract_path) / Path(file_path).stem
-    with output_path.open("wb") as output:
-        with gzip.open(file_path, "rb") as gz_archive:
-            shutil.copyfileobj(gz_archive, output)
-
-
-def _unpack_bz2(file_path: str, extract_path: str) -> None:
-    output_path = Path(extract_path) / Path(file_path).stem
-    with output_path.open(mode="wb") as output:
-        with bz2.open(file_path, "rb") as bz2_archive:
-            shutil.copyfileobj(bz2_archive, output)
-
-
-def _unpack_deb(file_path: str, extract_path: str) -> None:
-    ar_file = unix_ar.open(file_path)
-    file_names = [info.name.decode("utf-8") for info in ar_file.infolist()]
-    for file_name in file_names:
-        if file_name.startswith("debian-binary"):
-            continue
-        tarball = ar_file.open(file_name)
-        tar_file = tarfile.open(fileobj=tarball)
-        try:
-            tar_file.extractall(Path(extract_path) / file_name.split(".")[0])
-        finally:
-            tar_file.close()
-
-
-shutil.register_unpack_format("gz", [".gz"], _unpack_gz)
-shutil.register_unpack_format("bz2", [".bz2"], _unpack_bz2)
-shutil.register_unpack_format("deb", [".deb"], _unpack_deb)
-
-
 async def _download_file(url: str, download_path: Path) -> None:
     tmp_path = f"{download_path}.part"
     logger.info(f"Downloading {download_path.name}...")
@@ -86,32 +50,6 @@ async def _hash_file(file_path: Path) -> str:
     return sha256_hash.hexdigest()
 
 
-def is_archive_format_supported(archive_path: Path) -> bool:
-    for name, extensions, _ in shutil.get_unpack_formats():
-        for extension in extensions:
-            if archive_path.name.endswith(extension):
-                return True
-    return False
-
-
-async def extract_archive(archive_path: Path, extract_path: Path) -> None:
-    tmp_extract_path = f"{extract_path}_tmp"
-    Path(tmp_extract_path).mkdir(exist_ok=True)
-    logger.info(f"Extracting {archive_path.name}...")
-
-    try:
-        await asyncio.get_running_loop().run_in_executor(
-            None, shutil.unpack_archive, archive_path, tmp_extract_path
-        )
-    except Exception as e:
-        error = f"Failed to extract archive {archive_path}"
-        if str(e):
-            error += f" ({e})"
-        log_and_raise(Ops2debExtractError(error))
-
-    shutil.move(tmp_extract_path, extract_path)
-
-
 @dataclass
 class FetchResult:
     url: str
@@ -187,27 +125,18 @@ class Fetcher:
 
     def fetch_urls_and_check_hashes(
         self,
-        remote_files: Sequence[str | RemoteFile],
+        urls: Sequence[str],
     ) -> tuple[dict[str, FetchResult], dict[str, Ops2debError]]:
         tasks: list[FetchTask] = []
-        for remote_file in set(remote_files):
-            if isinstance(remote_file, str):
-                task = FetchTask(remote_file, self.lock.sha256(remote_file))
-            else:
-                # TODO: For backward compatibility, RemoteFile will soon be removed
-                task = FetchTask(remote_file.url, remote_file.sha256)
-            tasks.append(task)
+        for url in set(urls):
+            tasks.append(FetchTask(url, self.lock.sha256(url)))
         return asyncio.run(self._run_tasks(tasks))
 
     def update_lockfile(self, configuration_path: Path) -> None:
-        blueprints = extend(parse(configuration_path))
-
         urls: list[str] = []
-        for blueprint in blueprints:
-            url = blueprint.render_fetch_url()
-            if url is not None and url not in self.lock:
-                urls.append(url)
-
+        for blueprint in Configuration(configuration_path).blueprints:
+            fetch_urls = blueprint.render_fetch_urls()
+            urls.extend([url for url in fetch_urls if url not in self.lock])
         results, fetch_errors = asyncio.run(self.fetch_urls(urls))
         self.lock.add(list(results.values()))
         self.lock.save()
