@@ -1,6 +1,7 @@
 import logging
 import shutil
 import subprocess
+from itertools import product
 from pathlib import Path
 
 from dirsync import sync
@@ -9,10 +10,12 @@ from ops2deb import logger
 from ops2deb.apt import DebianRepositoryPackage, sync_list_repository_packages
 from ops2deb.exceptions import Ops2debGeneratorError, Ops2debGeneratorScriptError
 from ops2deb.fetcher import Fetcher, FetchResult
-from ops2deb.parser import Blueprint, HereDocument, SourceDestinationStr, extend
+from ops2deb.parser import Blueprint, HereDocument, SourceDestinationStr
 from ops2deb.templates import environment
 from ops2deb.utils import working_directory
 
+BASE_TEMPORARY_DIRECTORY = Path("/tmp/ops2deb_tmp")
+
 
 def _format_command_output(output: str) -> str:
     lines = output.splitlines()
@@ -34,8 +37,9 @@ class SourcePackage:
         self.debian_directory = self.package_directory / "debian"
         self.source_directory = self.package_directory / "src"
         self.fetch_directory = self.package_directory / "fetched"
+        self.temporary_directory = Path("/tmp/ops2deb_tmp") / self.directory_name
+        self.fetch_url = blueprint.render_fetch_url()
         self.blueprint = blueprint
-        self.remote_file = self.blueprint.render_fetch()
 
     def _render_template(self, template_name: str) -> None:
         template = environment.get_template(f"{template_name}")
@@ -49,6 +53,8 @@ class SourcePackage:
         shutil.rmtree(self.fetch_directory, ignore_errors=True)
         shutil.rmtree(self.source_directory, ignore_errors=True)
         self.source_directory.mkdir(parents=True)
+        shutil.rmtree(self.temporary_directory, ignore_errors=True)
+        self.temporary_directory.mkdir(parents=True)
         for path in ["usr/bin", "usr/share", "usr/lib", "etc"]:
             (self.source_directory / path).mkdir(parents=True)
 
@@ -69,9 +75,20 @@ class SourcePackage:
                 logger=dirsync_logger,
             )
 
+    def _render_string(self, string: str) -> str:
+        return self.blueprint.render_string(
+            string,
+            src=self.source_directory,
+            debian=self.debian_directory,
+            cwd=self.configuration_directory,
+            tmp=self.temporary_directory,
+        )
+
     def _install_here_document(self, entry: HereDocument, destination: Path) -> None:
         if destination.exists():
-            raise Ops2debGeneratorError(f"Destination {destination} already exists")
+            raise Ops2debGeneratorError(
+                f"Failed to write {destination}, file already exists"
+            )
         destination.write_text(self._render_string(entry.content))
 
     def _install_source_destination_str(
@@ -79,30 +96,25 @@ class SourcePackage:
     ) -> None:
         source = Path(self._render_string(entry.source))
         if source.exists() is False:
-            raise Ops2debGeneratorError(f"Source {str(source)} does not exist")
+            raise Ops2debGeneratorError(
+                f"Failed to copy {str(source)}, it does not exist"
+            )
         if source.is_dir() is True:
             shutil.copytree(source, destination, dirs_exist_ok=True, symlinks=True)
         elif source.is_file() is True:
             shutil.copy2(source, destination)
         else:
             raise Ops2debGeneratorError(
-                f"Source {str(source)} is not a file nor a directory"
+                f"Failed to copy {str(source)}, it is not a file nor a directory"
             )
 
-    def _render_string(self, string: str) -> str:
-        return self.blueprint.render_string(
-            string,
-            src=self.source_directory,
-            debian=self.debian_directory,
-            cwd=self.configuration_directory,
-        )
-
     def _install_files(self) -> None:
         for entry in self.blueprint.install:
             destination = Path(self._render_string(entry.destination))
             if (
                 destination.is_absolute() is True
                 and destination.is_relative_to(self.package_directory) is False
+                and destination.is_relative_to(self.temporary_directory) is False
             ):
                 destination = self.source_directory / destination.relative_to("/")
             if destination.is_absolute() is False:
@@ -128,9 +140,10 @@ class SourcePackage:
 
     def generate(self, fetch_results: dict[str, FetchResult]) -> None:
         fetch_result: FetchResult | None = None
-        if (url := self.blueprint.render_fetch_url()) is not None:
-            fetch_result = fetch_results.get(url, None)
+        if self.fetch_url is not None:
+            fetch_result = fetch_results.get(self.fetch_url, None)
             if fetch_result is None:
+                # we failed to download the archive needed to build this package
                 return
 
         logger.title(f"Generating source package {self.directory_name}...")
@@ -183,6 +196,16 @@ def filter_already_published_packages(
     return filtered_packages
 
 
+def extend(blueprints: list[Blueprint]) -> list[Blueprint]:
+    extended_list: list[Blueprint] = []
+    for blueprint in blueprints:
+        for arch, version in product(blueprint.architectures(), blueprint.versions()):
+            extended_list.append(
+                blueprint.copy(update={"architecture": arch, "version": version})
+            )
+    return extended_list
+
+
 def generate(
     fetcher: Fetcher,
     blueprints: list[Blueprint],
@@ -191,7 +214,7 @@ def generate(
     debian_repository: str | None = None,
     only_names: list[str] | None = None,
 ) -> list[SourcePackage]:
-    # each blueprint can yield multiple source packages, one per supported arch
+    # each blueprint can yield multiple source packages
     packages = [
         SourcePackage(b, output_directory, configuration_directory)
         for b in extend(blueprints)
@@ -205,8 +228,8 @@ def generate(
         packages = filter_already_published_packages(packages, debian_repository)
 
     # run fetch instructions (download, verify, extract) in parallel
-    files = [p.remote_file for p in packages if p.remote_file is not None]
-    fetch_results, fetch_errors = fetcher.fetch_urls_and_check_hashes(files)
+    urls = [p.fetch_url for p in packages if p.fetch_url is not None]
+    fetch_results, fetch_errors = fetcher.fetch_urls_and_check_hashes(urls)
 
     for package in packages:
         package.generate(fetch_results)
